from base64 import b64encode
from django.core.management.base import BaseCommand, CommandError
from optparse import make_option
import json
import urllib
import urllib2


class Command(BaseCommand):
    help = 'Synchronizes a geogit repository using the Geoserver-Geogit api.'
    args = '<arg>'
    option_list = BaseCommand.option_list + (
        make_option('-u', '--url',
                    dest='url',
                    default='http://example.com',
                    help='The url to the repository you want to sync.'),
        make_option('-d', '--direction',
                    dest='direction',
                    default='duplex',
                    help='The direction that you want to sync in (push, pull, or duplex).'),
        make_option('-r', '--remote',
                    dest='remote',
                    default='origin',
                    help='The remote that you want to sync with. Defaults to origin.'),
        make_option('--remotebranch',
                    dest='remoteBranch',
                    default='master',
                    help='The remote branch to sync. Defaults to master.'),
        make_option('--localbranch',
                    dest='localBranch',
                    default='master',
                    help='The local branch to sync. Defaults to master.'),
        make_option('--username',
                    dest='username',
                    default=None,
                    help='The username to use for basic auth requests.'),
        make_option('--password',
                    dest='password',
                    default=None,
                    help='The password to use for basic auth requests.'),
        make_option('--authorname',
                    dest='authorname',
                    default=None,
                    help='The author name to use in potential merge commits.'),
        make_option('--authoremail',
                    dest='authoremail',
                    default=None,
                    help='The author email to use in potential merge commits.'),
    )

    def make_request(self, url, params, auth=None):
        """
        Prepares a request from a url, params, and optionally authentication.
        """
        req = urllib2.Request(url + urllib.urlencode(params))

        if auth:
            req.add_header('AUTHORIZATION', 'Basic ' + auth)

        return urllib2.urlopen(req)

    def handle(self, *args, **options):

        url = options.get('url')

        if (url is None) or len(url) == 0:
            raise CommandError("The url specified is either missing or blank.")

        index = url.rfind('/')

        if index != (len(url)-1):
            url += '/'

        direction = options.get('direction')

        if not (direction == "push" or direction == "pull" or direction == "duplex"):
            raise CommandError("The sync direction must be either: push, pull, or duplex.")

        remote = options.get('remote')
        remoteBranch = options.get('remoteBranch')
        localBranch = options.get('localBranch')
        username = options.get('username')
        password = options.get('password')
        authorname = options.get('authorname')
        authoremail = options.get('authoremail')
        auth = None

        if username and password:
            auth = b64encode('{0}:{1}'.format(username, password))

        params = {'output_format': 'JSON', 'remoteName': remote, 'ping': 'true'}
        request = self.make_request(url=url+'remote?', params=params, auth=auth)
        pingSuccess = False
        if request.getcode() != 200:
            raise CommandError("The server that you are trying to sync seems to be down at the moment, "
                               "try again later.")
        try:
            response = json.loads(request.read())
            if 'ping' in response['response']:
                if response['response']['ping']['success']:
                    pingSuccess = True
        except ValueError:
            pass
        if not pingSuccess:
            raise CommandError("The remote that you are trying to sync with seems to be down at the moment, try again "
                               "later.")

        def endTransaction(cancel, transactionId):
            params = {'output_format': 'JSON', 'cancel': cancel, 'transactionId': transactionId}
            request = self.make_request(url=url+'endTransaction?', params=params, auth=auth)

            if request.getcode() != 200:
                raise CommandError("EndTransaction failed: Status Code {0}".format(request.getcode()))

            response = json.loads(request.read())

            if not response['response']['success']:
                raise CommandError("An error occurred on endTransaction: {0}".format(response['response']['error']))

        params = {'output_format': 'JSON'}
        self.stdout.write('Starting transaction...')
        request = self.make_request(url=url+'beginTransaction?', params=params, auth=auth)

        if request.getcode() != 200:
            raise CommandError("BeginTransaction failed: Status Code {0}".format(request.getcode()))
        response = json.loads(request.read())

        if not response['response']['success']:
            raise CommandError("An error occurred on beginTransaction: {0}".format(response['response']['error']))

        self.stdout.write('Transaction started')
        transactionId = response['response']['Transaction']['ID']

        if direction == "pull" or direction == "duplex":
            params = {'output_format': 'JSON',
                      'remoteName': remote,
                      'ref': remoteBranch+':'+localBranch,
                      'transactionId': transactionId,
                      'authorName': authorname,
                      'authorEmail': authoremail}

            self.stdout.write('Beginning pull...')
            request = self.make_request(url=url+'pull?', params=params, auth=auth)

            if request.getcode() != 200:
                endTransaction(True, transactionId)
                raise CommandError("Pull failed: Status Code {0}".format(request.getcode()))
            response = json.loads(request.read())
            if response['response']['success']:
                if 'Merge' in response['response']:
                    endTransaction(True, transactionId)
                    raise CommandError("There were conflicts on pull")
            else:
                endTransaction(True, transactionId)
                raise CommandError("An error occurred on pull: {0}".format(response['response']['error']))
            self.stdout.write('Pull completed')

        if direction == "push" or direction == "duplex":
            params = {'output_format': 'JSON',
                      'remoteName': remote,
                      'ref': localBranch+':'+remoteBranch,
                      'transactionId': transactionId}

            self.stdout.write('Beginning push...')
            request = self.make_request(url=url+'push?', params=params, auth=auth)

            if request.getcode() != 200:
                endTransaction(True, transactionId)
                raise CommandError("Push failed: Status Code {0}".format(request.getcode()))
            response = json.loads(request.read())

            if not response['response']['success']:
                endTransaction(True, transactionId)
                raise CommandError("An error occurred on push: {0}".format(response['response']['error']))

            self.stdout.write('Push completed')

        self.stdout.write('Finishing transaction...')
        endTransaction(False, transactionId)
        self.stdout.write('Transaction completed')



from django.conf import settings
from datetime import datetime
import os


# convert dict to object
class DictToObject(object):
    def __init__(self, d):
        self.__dict__['d'] = d

    def __getattr__(self, key):
        value = self.__dict__['d'][key]
        if isinstance(value, dict):
            return DictToObject(value)
        return value


def get_fileservice_dir():
    """
    example settings file
    FILESERVICE_CONFIG = {
        'store_dir': '/var/lib/geoserver_data/fileservice_store'
    }
    """
    conf = getattr(settings, 'FILESERVICE_CONFIG', {})
    dir = conf.get('store_dir', './fileservice_store')
    return os.path.normpath(dir) + os.sep


def get_fileservice_whitelist():
    conf = getattr(settings, 'FILESERVICE_CONFIG', {})
    return conf.get('types_allowed', [])


def get_fileservice_server_route_internal():
    conf = getattr(settings, 'FILESERVICE_CONFIG', {})
    return conf.get('server_route_internal', '/server_route_internal/')


def get_gallery_file():
    conf = getattr(settings, 'FILESERVICE_CONFIG', {})
    return conf.get('gallery_file', get_fileservice_dir() + '/gallery.gal')


def u_to_str(string):
    return string.encode('ascii', 'ignore')


def is_int_str(v):
    v = str(v).strip()
    return v == '0' or (v if v.find('..') > -1 else v.lstrip('-+').rstrip('0').rstrip('.')).isdigit()


def add_file_attribs(target_object, filename):
    if os.path.isfile(filename):
        stat = os.stat(filename)
        if stat:
            target_object['file_size'] = stat.st_size
            target_object['file_updated'] = datetime.fromtimestamp(stat.st_ctime)


def get_fileservice_files():
    return os.listdir(get_fileservice_dir())


def file_exists(filename):
    if os.path.isfile(get_filename_absolute(filename)):
        return True
    return False


def get_filename_absolute(filename):
    return '{}/{}'.format(get_fileservice_dir(), filename)


def get_fileservice_files_abs():
    return [get_filename_absolute(i) for i in get_fileservice_files()]



#########################################################################
#
# Copyright (C) 2012 OpenPlans
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################

from django.core.management.base import BaseCommand
from django.core.exceptions import FieldError, MultipleObjectsReturned
from geonode.maps.models import MapLayer
from geonode.layers.models import Layer
from django.conf import settings
from urlparse import urlsplit


class Command(BaseCommand):
    help = 'Update the IP of local map layers'

    @staticmethod
    def replaceNetLoc(old_url, new_netloc):
        old_netloc = urlsplit(old_url).netloc
        return old_url.replace(old_netloc, new_netloc)

    def handle(self, *args, **options):
        new_netloc = urlsplit(settings.OGC_SERVER['default']['PUBLIC_LOCATION']).netloc
        map_layers = MapLayer.objects.filter(local=True).exclude(ows_url__icontains=new_netloc)
        layers = Layer.objects.exclude(storeType='remoteStore')

        for maplayer in map_layers:
            new_url = self.replaceNetLoc(maplayer.ows_url, new_netloc)
            self.stdout.write('Changing map layer url from {0} to {1}. (Map Layer #{2})'
                              .format(maplayer.ows_url, new_url, maplayer.id))
            maplayer.ows_url = new_url
            maplayer.save()

        for layer in layers:
            for link in layer.link_set.all().exclude(url__isnull=True).exclude(url__icontains=new_netloc):
                new_url = self.replaceNetLoc(link.url, new_netloc)
                self.stdout.write('Updating the url for the "{0}" link on layer {1} from {2} to {3}'.format(
                    link.name, layer.name, link.url, new_url))
                link.url = self.replaceNetLoc(link.url, new_netloc)
                link.save()

        for layer in layers.exclude(distribution_url__isnull=True).exclude(distribution_url__icontains=new_netloc):
            new_url = self.replaceNetLoc(layer.distribution_url, new_netloc)
            self.stdout.write('Changing a layer distribution url from {0} to {1}. (Layer #{2})'.format(
                layer.distribution_url, new_url, layer.id))
            layer.distribution_url = new_url
            try:
                layer.save()
            except MultipleObjectsReturned:
                self.stdout.write('MultipleObjectsReturned Error when updating layer #{0}'.format(layer.id))

        try:
            for layer in layers.exclude(thumbnail_url__isnull=True).exclude(thumbnail_url__icontains=new_netloc):
                new_url = self.replaceNetLoc(layer.thumbnail_url, new_netloc)
                self.stdout.write('Changing a layer thumbnail url from {0} to {1}. (Layer #{2})'.format(
                    layer.thumbnail_url, new_url, layer.id))
                layer.thumbnail_url = new_url
                try:
                    layer.save()
                except MultipleObjectsReturned:
                    self.stdout.write('MultipleObjectsReturned Error when updating layer #{0}'.format(layer.id))
        except FieldError:
            self.stdout.write('The thumbnail_url field does not exist in this version of GeoNode, skipping.')



from django.contrib.sites.models import Site
from django.core.management.base import BaseCommand
from optparse import make_option


class Command(BaseCommand):
    help = 'Updates a Site object.'
    args = '<site id>'
    option_list = BaseCommand.option_list + (
        make_option('-s', '--site-id',
                    dest='site',
                    default='1',
                    help='The site id to update.  Defaults to 1.'),
        make_option('-d', '--domain',
                    dest='domain',
                    default='http://example.com',
                    help='The site\'s domain.  http://example.com'),
        make_option('-n', '--display-name',
                    dest='name',
                    default='example.com',
                    help='The display name of the site.'),
    )

    def handle(self, *args, **options):

        site_id = options.get('site')
        domain = args[0] if args else options.get('domain')
        name = options.get('name')

        site, created = Site.objects.get_or_create(pk=int(site_id), defaults=dict(domain=domain, name=name))
        site.save()
        Site.objects.clear_cache()

        self.stdout.write('Successfully updated site {0} to: {1}.'.format(site_id, site.domain))



from django.conf import settings
from django.utils.translation import ugettext as _
from geoshape.version import get_version


def security_warnings(request, PROXY_ALLOWED_HOSTS=()):
    """ Detects insecure settings and reports them to the client-side context. """

    warnings = []

    PROXY_ALLOWED_HOSTS = PROXY_ALLOWED_HOSTS or getattr(settings, 'PROXY_ALLOWED_HOSTS', ())

    if PROXY_ALLOWED_HOSTS and '*' in PROXY_ALLOWED_HOSTS:
        warnings.append(dict(title=_('Insecure setting detected.'),
                             description=_('A wildcard is included in the PROXY_ALLOWED_HOSTS setting.')))

    return dict(warnings=warnings)


def rogue(request):
    """ Returns the geoshape version """

    return dict(VERSION=get_version())


def fileservice(request):
    """ Returns url template that can be used by client to resolve path to endpoint that serves media """

    conf = getattr(settings, 'FILESERVICE_CONFIG', {})
    return dict(fileservice_url_template=conf.get('url_template', '/api/fileservice/{}/view'))



from django.contrib.auth import get_user_model
from django.core.urlresolvers import reverse
from django.test import Client, TestCase
from geoshape.core.context_processors import security_warnings
from lxml import etree

import logging
logger = logging.getLogger(__name__)

User = get_user_model()


class ROGUETests(TestCase):

    def setUp(self):
        self.username = 'admin'
        self.password = 'admin'
        self.user, created = User.objects.get_or_create(username=self.username, is_superuser=True)

        if created:
            self.user.set_password(self.password)
            self.user.save()

    def test_jpeg_decoder(self):
        """
        Ensure the jpeg decoder is installed.

        The jpeg decoder (libjpeg) is a django-avatar dependency that can be installed with your operating system's
        package installer.  This test case fails if the decoder is not installed.
        """

        try:
            from PIL.Image import core
            getattr(core, "jpeg_decoder")
        except IOError:
            self.fail("JPEG Decoder not installed.")
        except ImportError:
            pass

    @staticmethod
    def get_elements_from_xpath(content, xpath='//title'):
        html = etree.HTML(content)
        return html.xpath(xpath)

    def test_security_warnings(self):
        """
        Tests the security warnings context processor.
        """

        PROXY_ALLOWED_HOSTS = ('*',)
        warnings = security_warnings(None, PROXY_ALLOWED_HOSTS)
        self.assertDictEqual(warnings, {'warnings': [{'description': u'A wildcard is included in '
                                                                     u'the PROXY_ALLOWED_HOSTS setting.',
                                                      'title': u'Insecure setting detected.'}]})

        PROXY_ALLOWED_HOSTS = ('.openstreetmap.com',)
        warnings = security_warnings(None, PROXY_ALLOWED_HOSTS)
        self.assertDictEqual(warnings, {'warnings': []})

    def test_documentation_links(self):
        """
        Ensures that links to the documentation are included in the templates.
        """

        c = Client()
        c.login(username=self.username, password=self.password)
        resp = c.get(reverse('home'), follow=True)
        docs_links = self.get_elements_from_xpath(resp.content, xpath='//a[@href="/docs"]')
        self.assertTrue(len(docs_links) > 0)



from tastypie.authentication import BasicAuthentication, SessionAuthentication, MultiAuthentication
from tastypie.authorization import Authorization
from tastypie.utils import trailing_slash
from tastypie.bundle import Bundle
from tastypie.resources import Resource
from django.conf.urls import url
from django.views.static import serve
from django.utils.encoding import smart_str
from django.http import HttpResponse
from httplib import NOT_ACCEPTABLE
from tastypie import fields
import helpers
import os


class FileItem(object):
    name = ''


class FileItemResource(Resource):
    name = fields.CharField(attribute='name')

    class Meta:
        resource_name = 'fileservice'
        object_class = FileItem
        fields = ['name']
        include_resource_uri = False
        allowed_methods = ['get', 'post', 'put']
        list_allowed_methods = ['post']
        always_return_data = True
        authentication = MultiAuthentication(SessionAuthentication(), BasicAuthentication())
        authorization = Authorization()

    def determine_format(self, request):
        return 'application/json'

    @staticmethod
    def get_file_items():
        file_names = helpers.get_fileservice_files()
        file_items = []
        for name in file_names:
            file_item = FileItem()
            file_item.name = name
            file_items.append(file_item)
        return file_items

    @staticmethod
    def get_file_item(kwargs):
        if 'name' in kwargs:
            return FileItemResource.get_file_by_name(kwargs['name'])
        elif 'pk' in kwargs:
            return FileItemResource.get_file_items()[int(kwargs['pk'])]
        return None

    @staticmethod
    def get_file_by_name(name):
        file_items = FileItemResource.get_file_items()
        for file_item in file_items:
            if file_item.name == helpers.u_to_str(name):
                return file_item

    def deserialize(self, request, data, format=None):
        if not format:
            format = request.META.get('CONTENT_TYPE', 'application/json')

        if format == 'application/x-www-form-urlencoded':
            return request.POST

        if format.startswith('multipart'):
            data = request.POST.copy()
            data.update(request.FILES)
            return data

        return super(Resource, self).deserialize(request, data, format)

    def detail_uri_kwargs(self, bundle_or_obj):
        if isinstance(bundle_or_obj, Bundle):
            return {'name': bundle_or_obj.obj.name}
        else:
            return {'name': bundle_or_obj.name}

    def get_object_list(self, request):
        # inner get of object list... this is where you'll need to
        # fetch the data from what ever data source
        return FileItemResource.get_file_items()

    def obj_get_list(self, request=None, **kwargs):
        # outer get of object list... this calls get_object_list and
        # could be a point at which additional filtering may be applied
        return self.get_object_list(request)

    def obj_get(self, request=None, **kwargs):
        # get one object from data source
        file_item = FileItemResource.get_file_item(kwargs)
        # if not file_item: raise NotFound("Object not found")
        return file_item

    def obj_create(self, bundle, request=None, **kwargs):
        # create a new File
        bundle.obj = FileItem()
        # full_hydrate does the heavy lifting mapping the
        # POST-ed payload key/values to object attribute/values
        bundle = self.full_hydrate(bundle)
        filename_name, file_extension = os.path.splitext(bundle.data[u'file'].name)

        # -- only allow uploading of files of types specified in FILESERVICE_CONFIG.types_allowed
        types_allowed = helpers.get_fileservice_whitelist()
        if '*' not in types_allowed and file_extension not in types_allowed:
            return HttpResponse(status=NOT_ACCEPTABLE, content='file type is not whitelisted in'
                                                               ' FILESERVICE_CONFIG.types_allowed')

        file_data = bundle.data[u'file'].read()
        # TODO: support optional unique name generation from sha1 and uuid.
        #  file_sha1 = hashlib.sha1(file_data).hexdigest() # is file_data only the bytes without filename etc?
        #  if file_extension:
        #    filename_name = '{}{}'.format(file_sha1, file_extension)
        #  else:
        #    filename_name = file_sha1
        bundle.obj.name = bundle.data[u'file'].name
        with open(helpers.get_filename_absolute(bundle.data[u'file'].name), 'wb+') as destination_file:
            destination_file.write(file_data)

        # remove the file object passed in so that the response is more concise about what this file will be referred to
        bundle.data.pop(u'file', None)
        return bundle

    def prepend_urls(self):
        """ Add the following array of urls to the resource base urls """
        return [
            url(r"^(?P<resource_name>%s)/(?P<pk>\w[\w/-]*)/download%s$" % (self._meta.resource_name, trailing_slash()),
                self.wrap_view('download'), name="api_fileitem_download"),
            url(r"^(?P<resource_name>%s)/(?P<name>[\w\d_.-]+)/download%s$"
                % (self._meta.resource_name, trailing_slash()), self.wrap_view('download'),
                name="api_fileitem_download"),
            url(r"^(?P<resource_name>%s)/(?P<pk>\w[\w/-]*)/view%s$"
                % (self._meta.resource_name, trailing_slash()), self.wrap_view('view'), name="api_fileitem_view"),
            url(r"^(?P<resource_name>%s)/(?P<name>[\w\d_.-]+)/view%s$"
                % (self._meta.resource_name, trailing_slash()), self.wrap_view('view'), name="api_fileitem_view"),
            url(r"^(?P<resource_name>%s)/(?P<name>[\w\d_.-]+)/$"
                % self._meta.resource_name, self.wrap_view('dispatch_detail'), name="api_dispatch_detail_name"),
            url(r"^(?P<resource_name>%s)/(?P<id>[\d]+)/$"
                % self._meta.resource_name, self.wrap_view('dispatch_detail'), name="api_dispatch_detail"),
        ]

    def download(self, request, **kwargs):
        # method check to avoid bad requests
        self.method_check(request, allowed=['get'])
        # Must be done otherwise endpoint will be wide open
        self.is_authenticated(request)

        response = None
        file_item = FileItemResource.get_file_item(kwargs)
        if file_item:
            filename_absolute = helpers.get_filename_absolute(file_item.name)
            if os.path.isfile(filename_absolute):
                response = serve(request, os.path.basename(filename_absolute), os.path.dirname(filename_absolute))
                response['Content-Disposition'] = 'attachment; filename="{}"'.format(
                    os.path.basename(filename_absolute))

        if not response:
            response = self.create_response(request, {'status': 'filename not specified'})

        return response

    def view(self, request, **kwargs):
        '''
        allow a file to be viewed as opposed to download. This is particularly needed when a video file is stored
        in the fileservice and user wants to be able to use a view the video as opposed to having to download it
        first. It passes the serving of the file to nginx/apache which will return all the proper headers allowing,
        say, html5's video viewer's 'seek' indicator/knob to work. Otherwise the video is only played sequentially

        Note that nginx/apache need to be configured accordingly. nginx for example:
        location /var/lib/geoserver_data/file-service-store/ {
           # forces requests to be authorized
           internal;
           alias   /var/lib/geoserver_data/file-service-store/;
        }

        for apache, need to install xsendfile module, enable it, set the path and then
        XSendFile on
        XSendFilePath /var/lib/geoserver_data/file-service-store

        example use:
        http://<ip>/api/v1/fileservice/med.mp4/view/
        '''
        # method check to avoid bad requests
        self.method_check(request, allowed=['get'])
        # Must be done otherwise endpoint will be wide open
        self.is_authenticated(request)

        response = None
        file_item = FileItemResource.get_file_item(kwargs)
        if file_item:
            # set content_type to '' so that content_type from nginx/apache is returned
            response = HttpResponse(content_type='')
            # needed for apache, re-test for nginx as nginx needed content_type='' but apache doesn't like that.
            # looks like module called django-sendfile exists which is worth considering if there are any issues here.
            del response['content-type']
            file_with_route = smart_str('{}{}'.format(helpers.get_fileservice_dir(), file_item.name))
            # apache header
            response['X-Sendfile'] = file_with_route
            # nginx header
            response['X-Accel-Redirect'] = file_with_route

        if not response:
            response = self.create_response(request, {'status': 'filename not specified'})

        return response



from setuptools import setup, find_packages

install_requires = [
        "geoshape-geonode==1.4",
        "django-classification-banner>=0.1.4",
        "django-maploom==1.4.0",
        "psycopg2==2.4.5",
        "django-tilebundler==0.1-beta1",
        "django-gsschema==0.1-beta2",
        "django-cors-headers>=1.1.0",
]

tests_requires = [
    "lxml==3.3.1",
]

docs_requires = [
    "sphinx-rtd-theme==0.1.5",
    "Sphinx==1.2.2",
]

setup(
    name="geoshape",
    version=__import__('geoshape').get_version(),
    author="Prominent Edge",
    author_email="geoshape.org@gmail.com",
    description="geoshape, based on GeoNode and contains additional changes and services",
    #long_description=(read('README.md')),
    classifiers=[
        'Development Status :: 5 - Production/Stable',
    ],
    license="BSD",
    keywords="geoshape geonode django",
    url='https://github.com/ROGUE-JCTD/rogue_geonode',
    packages=find_packages('geoshape'),
    package_dir={'': 'geoshape'},
    include_package_data=True,
    install_requires=install_requires,
    extras_require={
        'tests': install_requires + tests_requires,
        'docs':  docs_requires
    },
    zip_safe=False,
)
